@@ -169,8 +169,9 @@ getprogname (void)
 #include <fcntl.h>
 #include <unistd.h>
 
+#ifndef PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
 void
-p11_mutex_init (p11_mutex_t *mutex)
+p11_recursive_mutex_init (p11_mutex_t *mutex)
 {
 	pthread_mutexattr_t attr;
 	int ret;
@@ -181,6 +182,7 @@ p11_mutex_init (p11_mutex_t *mutex)
 	assert (ret == 0);
 	pthread_mutexattr_destroy (&attr);
 }
+#endif
 
 char *
 p11_dl_error (void)
@@ -268,6 +270,7 @@ p11_dl_error (void)
 {
 	DWORD code = GetLastError();
 	LPVOID msg_buf;
+	char *result;
 
 	FormatMessageA (FORMAT_MESSAGE_ALLOCATE_BUFFER |
 	                FORMAT_MESSAGE_FROM_SYSTEM |
@@ -276,7 +279,9 @@ p11_dl_error (void)
 	                MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
 	                (LPSTR)&msg_buf, 0, NULL);
 
-	return msg_buf;
+	result = strdup (msg_buf);
+	LocalFree (msg_buf);
+	return result;
 }
 
 int
@@ -523,7 +528,10 @@ strconcat (const char *first,
 	for (arg = first; arg; arg = va_arg (va, const char*)) {
 		size_t old_length = length;
 		length += strlen (arg);
-		return_val_if_fail (length >= old_length, NULL);
+		if (length < old_length) {
+			va_end (va);
+			return_val_if_reached (NULL);
+		}
 	}
 
 	va_end (va);
@@ -805,7 +813,7 @@ getauxval (unsigned long type)
 		extern int __libc_enable_secure;
 		secure = __libc_enable_secure;
 
-#elif defined(HAVE_ISSETUGID)
+#elif defined(HAVE_ISSETUGID) && defined(HAVE_ISSETUGID_OPENBSD)
 		secure = issetugid ();
 
 #elif defined(OS_UNIX)
@@ -910,6 +918,7 @@ fdwalk (int (* cb) (void *data, int fd),
 	struct rlimit rl;
 #endif
 
+#ifdef __linux__
 	dir = opendir ("/proc/self/fd");
 	if (dir != NULL) {
 		while ((de = readdir (dir)) != NULL) {
@@ -932,6 +941,7 @@ fdwalk (int (* cb) (void *data, int fd),
 		closedir (dir);
 		return res;
 	}
+#endif
 
 	/* No /proc, brute force */
 #ifdef HAVE_SYS_RESOURCE_H
